C
C-----------------------------------------------------------------------
C  FFT routines for Cray systems using libsci routines
C-----------------------------------------------------------------------
C
      SUBROUTINE SEIFTM(N,MAG,N2)
C  RETURN FOURIER TRANSFORM MAGNITUDE AND POWER OF 2 .GE. N
C  MAX LENGTH 65536
      INTEGER N,MAG,N2
      MAG = 0
      N2 = 0
      IF (N.EQ.0) RETURN 
      N2 = 1
  10  IF (N2.GE.N .OR. N.EQ.65536) RETURN
          N2 = 2*N2
          MAG = MAG+1
          GOTO 10
      RETURN
      END
C
C-----------------------------------------------------------------------
C
      SUBROUTINE SEIMRCF( LDA, M, A, LDB, B, N2, NS, NF, SA )
C
C  Perform multiple real - to - complex forward fourier transforms
C
C  lda - leading dimension of input data to be transformed
C  m - second dimension of input/output arrays (number of transforms)
C  a(lda,m) - input real array
C  ldb - leading dimension of output complex transform array.
C  b(ldb,m) - output complex array
C  n2 - transform length (must be .ge. ns)
C  ns - number of input samples. a(ns+1:lda,1:m) is ignored.
C  nf - number of output frequencies. b(nf+1:ldb,1:m) is undefined.
C  sa - work array, length n2 + 2
C
	real a(lda,m), sa(*)
	complex b(ldb,m)
C
C  Loop over input traces
	nz = n2-ns
	do j=1,m
C  Copy to work array
		call scopy( ns, a(1,j), 1, sa, 1 )
C  Zero fill
		if (nz.gt.0) call vfill( 0.0, sa(ns+1), 1, nz )
C  Forward transform
		call seircft( n2, sa )
C  Copy to output array
		call ccopy( nf, sa, 1, b(1,j), 1 )
	enddo
C
	return
	end
C
C-----------------------------------------------------------------------
C
      SUBROUTINE SEIMCRF( LDA, M, A, LDB, B, N2, NS, NF, SA )
C
C  Perform multiple complex - to - real inverse fourier transforms
C
C  lda - leading dimension of output real inverse transform array
C  m - second dimension of input/output arrays (number of transforms)
C  a(lda,m) - output real array array
C  ldb - leading dimension of input complex transform array.
C  b(ldb,m) - input complex array
C  n2 - transform length (must be .ge. ns)
C  ns - number of output samples. a(ns+1:lda,1:m) is undefined.
C  nf - number of frequencies defined in b. b(nf+1:ldb,1:m) is ignored.
C  sa - work array, length n2 + 2
C
	real a(lda,m)
	complex b(ldb,m), sa(*)
C
C  Nyquist sample
	nyq = n2/2 + 1
	nz = 2*(nyq-nf)
C
C  Loop over input traces
	do j=1,m
C  Copy to work array
		call ccopy( nf, b(1,j), 1, sa, 1 )
C  Zero fill
		if (nz.gt.0) call vfill( 0.0, sa(nf+1), 1, nz )
C  Inverse transform
		call seicrft( n2, sa )
C  Copy to output array
		call scopy( ns, sa, 1, a(1,j), 1 )
	enddo
C
	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine seicft( n2, z, isign )
C
C  In-place forward transform, complex to complex
C
C  n2 - input transform length, must be power of 2
C  z - input/output transform array
C  isign - exponential sign, -1 for forward, +1 for inverse
C
	implicit none
	integer n2, isign
	complex z(*)
	real scale
	include 'crayfft.inc'

	if (n2.gt.N2MAX) then
		write (*,*) 'crayfft: length too long'
		return
	endif

C  Initialize if needed
	if (n2.ne.kn2 .or. ktype.ne.1) then
		ktype = 1
		kn2 = n2
		call cfft2( 1, isign, kn2, z, work, temp )
	endif

C  Forward transform
	call cfft2( 0, isign, kn2, z, work, temp )

C  Scale factor (on inverse only)
	if (isign.eq.1) then
		scale = 1./float(n2)
		call vsmul( temp, 1, scale, z, 1, 2*n2 )
	else
		call ccopy( n2, temp, 1, z, 1 )
	endif

	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine seircft( n2, z )
C
C  In-place forward transform, real to complex
C
C  n2 - input transform length, must be power of 2
C  z - input/output transform array
C
	implicit none
	integer n2
	complex z(*)
	include 'crayfft.inc'
	integer ISIGN
	parameter (ISIGN=-1)

	if (n2.gt.N2MAX) then
		write (*,*) 'crayfft: length too long'
		return
	endif

	if (n2.ne.kn2 .or. ktype.ne.2) then
		ktype = 2
		kn2 = n2
		call rcfft2( 1, ISIGN, kn2, z, work, temp )
	endif

C  Forward transform
	call rcfft2( 0, ISIGN, kn2, z, work, temp )

C  Copy back to input
	call ccopy( kn2/2 + 1, temp, 1, z, 1 )
	
	return
	end
C
C-----------------------------------------------------------------------
C
	subroutine seicrft( n2, z )
C
C  In-place inverse transform, complex to real
C
C  n2 - input transform length, must be power of 2
C  z - input/output transform array
C
	implicit none
	integer n2
	complex z(*)
	include 'crayfft.inc'
	real scale
	integer ISIGN
	parameter (ISIGN=1)

	if (n2.gt.N2MAX) then
		write (*,*) 'crayfft: length too long'
		return
	endif

	if (n2.ne.kn2 .or. ktype.ne.2) then
		ktype = 2
		kn2 = n2
		call crfft2( 1, ISIGN, kn2, z, work, temp )
	endif

C  Inverse transform
	call crfft2( 0, ISIGN, kn2, z, work, temp )

C  Scale on inverse
	scale = 1./float(2*kn2)
	call vsmul( temp, 1, scale, z, 1, kn2 )
	
	return
	end
C
C*----------------------------------------------------------------------
C*
C*  NAME:
C*    seift2d.f - 2D Fourier transforms for real data
C*     
C*  SYNOPSIS:
C*    integer n2, m2, lda, ldb, nf, ns, ntri 
C*    real a(lda,ntri), sa(n2+2)
C*    complex b(ldb,m2)
C*
C*    call seif2rc( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C*    call seif2cr( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C*
C*  DESCRIPTION: 
C*    seif2rc performs the forward 2D fft,
C*    seif2cr performs the inverse.
C*
C*----------------------------------------------------------------------
C*
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*----------------------------------------------------------------------
C
	subroutine seif2rc( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C
C  Forward 2D Fourier Transform for real data
C
C  n2, m2 - transform lengths
C  lda - leading dimension of input array
C  a(lda,*) - input real array containing data to be transformed
C  ldb - leading dimension of output array
C  b(ldb,m2) - output complex transform
C  nf - number of frequencies to compute
C  ns - number of input samples per trace 
C  ntri - number of input traces
C  sa - work array, length n2+2
C
	implicit none
C  Argument declarations
	integer n2, m2, lda, ldb, nf, ns, ntri
	real a(lda,*), sa(*)
	complex b(*)
C  Local declarations
	integer i,j,k,nz

C  Number of points to be zeroed on input
	nz = n2 - ns

C  Loop over number of traces and forward transform
	do j=1,ntri
C  Move data to work array, pad with zeros, and transform
		call scopy( ns, a(1,j), 1, sa, 1 )
		if (nz.gt.0) call vfill( 0.0, sa(ns+1), 1, nz )
		call seircft( n2, sa )
C  Move to output array
		k = ldb*(j-1) + 1
		call ccopy( nf, sa, 1, b(k), 1 )
	enddo

C  Zero pad remainder of output array with zeros
	nz = 2*ldb*(m2-ntri)
	k = ldb*ntri + 1
	if (nz.ne.0) call vfill( 0.0, b(k), 1, nz )

C  Transpose 
	call tran21c( ldb, m2, b )

C  Loop and perform second set of transforms
	do j=1,nf
		k = m2*(j-1) + 1
		call seicft( m2, b(k), 1 )
	enddo

C  Transpose back
	call tran21c( m2, ldb, b )

	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine seif2cr( n2, m2, lda, a, ldb, b, nf, ns, ntri, sa )
C
C  Inverse 2D Fourier Transform for real data
C
C  n2, m2 - transform lengths
C  lda - leading dimension of output array
C  a(lda,ntri) - output real array containing inverse transformed data
C  ldb - leading dimension of input array
C  b(ldb,m2) - input complex transform
C  nf - number of frequencies to compute
C  ns - number of output samples per trace 
C  ntri - number of output traces
C  sa - work array, length n2+2
C
	implicit none
C  Argument declarations
	integer n2, m2, lda, ldb, nf, ns, ntri
	real a(lda,*)
	complex b(*), sa(*)
C  Local declarations
	integer i,j,k,nz

C  Transpose 
	call tran21c( ldb, m2, b )

C  Loop and perform inverse transform across traces
	do j=1,nf
		k = m2*(j-1) + 1
		call seicft( m2, b(k), -1 )
	enddo

C  Transpose back
	call tran21c( m2, ldb, b )

C  Number of points to be zeroed in the frequency domain
	nz = 2*(n2/2 + 1 - nf)

C  Loop and inverse transform over samples
	do j=1,ntri
C  Move data to work array, pad with zeros, and inverse transform
		k = ldb*(j-1) + 1
		call ccopy( nf, b(k), 1, sa, 1 )
		if (nz.gt.0) call vfill( 0.0, sa(nf+1), 1, nz )
		call seicrft( n2, sa )
C  Move to output array
		call scopy( ns, sa, 1, a(1,j), 1 )
	enddo

	return
	end
C
C*----------------------------------------------------------------------
