C*----------------------------------------------------------------------
C*
C*  NAME:
C*    dgenp, dgena, dgenb, dgenc - Seismic Benchmark process DGEN
C*     
C*  SYNOPSIS:
C*    integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
C*    real otr(ldim,maxtrc), ra(nra), sa(nra)
C*    logical abort
C*    call dgenp( ldim, maxtrc, nra, nsa, abort, ipr )
C*    call dgena( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                abort, ipr )
C*    call dgenb( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntri, ntro, abort, ipr )
C*    call dgenc( ldim, maxtrc, otr, nra, ra, nsa, sa,
C*                ntro, abort, ipr )
C*
C*  DESCRIPTION: 
C*    Generates synthetic seismic data
C*
C*  PARAMETERS: 
C*    Record 1
C*    Field  Columns   Description
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*     Include files:
C*       ./coord.inc
C*       ./group.inc
C*       ./dgen.inc
C*
C*     Libraries:
C*       libbench.a
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*      References:
C*
C*      Modified:
C*
C*----------------------------------------------------------------------

C*----------------------------------------------------------------------
C*   NAME:
C*     dgenp
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   This process reads 'DGEN' parameters, checks syntax, and sets
C*   memory requirements for the processing phase
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*      Subroutines:
C*        getparm - read parameter record
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/06/90 - C. Mosher
C*
C*      References:
C*        SPARC System Documentation
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine dgenp( ldim, maxtrc, nra, nsa, abort, ipr )
C
C  Arguments:
C
C  Output leading dimension of trace array required for the PROC step
	integer LDIM
C  Output max number of traces per group to be output during PROC step
	integer MAXTRC
C  Number of words of reserved storage required in PROC step
	integer NRA
C  Number of words of working storage required in PROC step
	integer NSA
C  Abort flag, set to .TRUE. by this process if errors detected
	logical ABORT
C  Print unit number that this routine should use
	integer IPR

C
C  Common blocks:
C
C  parameters for process DGEN
	include 'dgen.inc'
C  data structure definition
	include 'group.inc'

C
C  Local declarations:
C
C  parameter record buffer
	character*80 buf
C  parameter record index
	integer index

C
C  Start of executable statements
C

C  Read first card
	index = 1
	call getparm('DGEN',index,buf,ier)
	if (ier.ne.0) goto 999
	read (buf,'(7x,i3,f10.0,i5)') idbug,vel,nref
C
	write (ipr,9010) vel,nref
 9010 format (/,' DGEN Model Parameters',/,
     +          ' Velocity          : ',f10.1,/,
     +          ' Number of events  : ',i5)
C
C  Read wavelet parms
	index = index + 1
	call getparm('DGEN',index,buf,ier)
	if (ier.ne.0 .or. buf(8:10).ne.'WVL') goto 999
	read (buf,'(10x,6f5.0)') flc,fl,fh,fhc,gap,gamp
	write (ipr,9020) flc,fl,fh,fhc
 9020 format (' Wavelet shape     : ',4f7.1)
	if (gap.ne.0.) write (ipr,9025) gap,gamp
 9025 format (' Ghost gap, amp    : ',2f7.1)
	gap = 0.001*gap
C  Compute fourier transform magnitude and length
	call seiftm(ns,mag,n2)
C  pad for wrap-around
	if (float(ns)/float(n2) .gt. 0.8) mag = mag + 1
	n2 = 2**mag
	nyq = n2/2 + 1
C  Set sample frequency
	sf = 1000./float(n2*msr)
	nf = 1 + nint(fhc/sf)
	write (ipr,9030) n2,sf
 9030 format (' Transform length  : ',i5,/,
     +		  ' Sample frequency  : ',f7.4,//,
     +        ' Event  Type         X         Y         Z       Ref')
C
C  Loop and read events

	do 10 j=1,nref
		index = index + 1
		call getparm('DGEN',index,buf,ier)
		if (ier.ne.0 .or. buf(8:10).ne.'REF') goto 999
		read (buf,'(10x,4f10.0,i5)') (xn(i,j),i=1,3),ref(j),ityp(j)
		write (ipr,'(2i6,4f10.1)') j,ityp(j),(xn(i,j),i=1,3),ref(j)
		if (ityp(j).eq.0) then
			if (xn(1,j).eq.0.) xn(1,j) = vel
		else if (ityp(j).eq.2) then
			call setnrm(xn(1,j),z0(j))
		endif
  10  continue

C
C  Set memory requirements
	ldim = ns
	maxtrc = ntpg
	nra = n2+2 + 4*nref
	nra = max(nra,2*n2)
	nsa = 2*n2+4 + 2*nref
	nsa = max( nsa, 3*nf )
C
	return
C
C  Error exit
C
 999  continue
	write (ipr,'(/,'' *** Model syntax error'')')
	if (ier.ne.0) ier = 1
	return
	end

C*----------------------------------------------------------------------
C*   NAME:
C*     dgena, dgenb, dgenc
C*     
C*----------------------------------------------------------------------
C*   DESCRIPTION:
C*
C*   Processing phase routines for data generation
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        01/01/92 - C. Mosher
C*
C*      References:
C*        Seismic Benchmark Users Guide
C*
C*      Modified:
C*
C*----------------------------------------------------------------------

C*----------------------------------------------------------------------
C*  The 'a' routine initializes wavelet
C*----------------------------------------------------------------------

	subroutine dgena( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		abort, ipr )
	integer ldim, maxtrc, nra, nsa, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	include 'para.inc'
	include 'dgen.inc'

C  Set debug print unit
	if (idbug.ne.0 .and. node.eq.master ) then
		idbug = ipr
	else
		idbug = 0
	endif

C  Store wavelet spectrum
	call dgenamp(nyq,sf,flc,fl,fh,fhc,sa)
	if (idbug.ne.0) write (ipr,9010) (sa(i),i=1,nyq)
 9010 format (' DGEN - Wavelet spectrum: ',/,(5g12.4))
	call dgengho(nyq,sf,gap,gamp,sa,ra)

C  Amplitude summary initialization flag
	maxini = .true.

	return
	end

C*----------------------------------------------------------------------
C*  The 'b' routine generates a synthetic shot record
C*----------------------------------------------------------------------

	subroutine dgenb( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntri, ntro, abort, ipr )
	integer ldim, maxtrc, nra, nsa, ntri, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	include 'dgen.inc'
	include 'group.inc'
	include 'coord.inc'

C  Pointers into work and reserved arrays
	k1 = n2 + 3
	k2 = k1+nref
	k3 = k2+nref
	k4 = k3+nref

	if (idbug.ne.0) 
     +  write (ipr,'(/,'' DGEN Debug: Group '',i5)') igroup

C  Loop over number of traces
	ntro = ntri
	do j=1,ntro
C  Generate trace
		if (idbug.ne.0) write (ipr,9010) j
 9010   format(' Trace ',i3,' Time, Amplitude pairs: ')
		call gentrc( sxyz(1,j), rxyz(1,j), ra, sa(k1), sa(k2), 
     +		ra(k1), ra(k2), ra(k3), ra(k4), sa, sa(k3), idbug )
		do i=1,ns
			otr(i,j) = sa(i)
		enddo
	enddo

	return
	end

C*----------------------------------------------------------------------
C*  'c' routine prints event summary
C*----------------------------------------------------------------------
c
	subroutine dgenc( ldim, maxtrc, otr, nra, ra, nsa, sa, 
     +		ntro, abort, ipr )
	integer ldim, maxtrc, nra, nsa, ntro, ipr
	real otr(ldim,maxtrc), ra(nra), sa(nsa)
	logical abort

	include 'dgen.inc'
	include 'para.inc'

	if (node.ne.master) return

C  Pointers into work and reserved arrays
	k1 = n2+2
	k2 = k1+nref
	k3 = k2+nref
	k4 = k3+nref

	write (ipr,9010) (i,ra(k1+i),ra(k2+i),ra(k3+i),ra(k4+i),i=1,nref)
 9010 format (' Process DGEN completed, Event Summary:',//,
     +        ' Event   Min Time    Max Time     Min Amp     Max Amp',/,
     +        (i6,4g12.4))

	return
	end

C*----------------------------------------------------------------------
C*  Support subroutines
C*----------------------------------------------------------------------

	subroutine gentrc( sxyz, rxyz, amp, dt, wt, tmin, tmax, 
     +	xmin, xmax, otr, sa, ipr ) 
C
C  Generate a seismic trace
C  real sxyz(3) - input source coords
C  real rxyz(3) - input receiver coords
C  real amp(*) - input wavelet spectrum
C  real dt(*),wt(*) - output event times and amplitudes
C  real tmin(*),tmax(*) - input/output time range for each event
C  real xmin(*),xmax(*) - input/output amplitude range for each event
C  real otr(*) - output synthetic trace
C  real sa(*) - work array, length n2+2
C  integer ipr - debug print unit, 0 for no print
C  
	real sxyz(3),rxyz(3),dt(*),wt(*),xmin(*),xmax(*),tmin(*),tmax(*)
	real amp(*),otr(*),sa(*)
	integer ipr

C  Event parameters
	include 'dgen.inc'

C  Loop over number of events and compute delay times
	do k=1,nref
C
C  ityp = 0: direct arrival
		if (ityp(k).eq.0) then
			call dirta(k,sxyz,rxyz,dt(k),wt(k))
C  ityp = 1: diffractor
		else if (ityp(k).eq.1) then
			call difta(k,sxyz,rxyz,dt(k),wt(k))
C  ityp = 2: reflector
		else if (ityp(k).eq.2) then
			call refta(k,sxyz,rxyz,dt(k),wt(k))
C  ityp = 3: spike at a given location
		else 
			call spkta(k,sxyz,rxyz,dt(k),wt(k))
		endif
C  Set min,max values for amplitude and time
		if (maxini) then
			xmin(k) = wt(k)
			xmax(k) = wt(k)
			tmin(k) = dt(k)
			tmax(k) = dt(k)
		endif
		xmin(k) = min(wt(k),xmin(k))
		xmax(k) = max(wt(k),xmax(k))
		tmin(k) = min(dt(k),tmin(k))
		tmax(k) = max(dt(k),tmax(k))
	enddo
	maxini = .false.

C  Sum phase shifted wavelets
	call syntrc(n2,mag,nf,sf,amp,nref,dt,wt,otr,sa)
	if (ipr.ne.0) then
		write (ipr,9010) (dt(i),wt(i),i=1,nref)
 9010	format ( '(',2g11.4,')','(',2g11.4,')','(',2g11.4,')' )
	endif
	return
	end

C*----------------------------------------------------------------------

	subroutine dirta(k,sxy,rxy,dt,wt)
C  Comput direct arrival travel time and amplitude weight
C  integer k - event number
C  real sxy(3) - input source location
C  real rxy(3) - input receiver location
C  real dt - output travel time in seconds
C  real wt - output amplitude weight

	real sxy(3),rxy(3),dt,wt
	include 'dgen.inc'
	r = dist(sxy,rxy)
	if (r.gt.0.) then
		dt = r/xn(1,k)
		wt = 1000./r
	else
		dt = 0.0
		wt = 0.0
	endif
	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine difta(k,sxy,rxy,dt,wt)
C  Comput diffraction travel time and amplitude weight
C  integer k - input event number
C  real sxy(3) - input source location
C  real rxy(3) - input receiver location
C  real dt - output travel time in seconds
C  real wt - output amplitude weight
	dimension sxy(3),rxy(3)
	include 'dgen.inc'
	r = dist(sxy,xn(1,k)) + dist(xn(1,k),rxy)
	if (r.gt.0.) then
		dt = r/vel
		wt = 1000.*ref(k)/r
	else
		dt = 0.0
		wt = 0.0
	endif
	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine refta(k,sxy,rxy,dt,wt)
C  Comput reflection travel time and amplitude weight
C  integer k - input event number
C  real sxy(3) - input source location
C  real rxy(3) - input receiver location
C  real dt - output travel time in seconds
C  real wt - output amplitude weight
	dimension sxy(3),rxy(3),xv(3)
	include 'dgen.inc'
	call virsrc(sxy,rxy,xn(1,k),z0(k),xv,ier)
	if (ier.eq.0) then
		r = dist(xv,rxy)
		dt = r/vel
		wt = 1000.*ref(k)/r
	else
		dt = -1.0
		wt = 0.0
	endif
	return
	end
C
C*----------------------------------------------------------------------
C
	function dist(x,y)
C  compute distance between points
	dimension x(3),y(3)
	dist = sqrt( (x(1)-y(1))**2 + (x(2)-y(2))**2 + (x(3)-y(3))**2 )	
	return
	end

C*----------------------------------------------------------------------

	subroutine spkta(k,sxy,rxy,dt,wt)
C  Determine if spike exists for this trace
C  integer k - event number
C  real sxy(3) - input source location
C  real rxy(3) - input receiver location
C  real dt - output travel time in seconds
C  real wt - output amplitude weight

	real sxy(3),rxy(3),dt,wt
	include 'dgen.inc'
	parameter (EPS = 0.1)

	dt = 0.0
	wt = 0.0

C  Check to see if source and receiver match spike location
	r1 = sqrt( (sxy(1)-xn(1,k))**2 + (sxy(2)-xn(2,k))**2 )
	r2 = sqrt( (rxy(1)-xn(1,k))**2 + (rxy(2)-xn(2,k))**2 )
	if (r1.gt.eps .or. r2.gt.eps) return

C  Set amplitude and location
	wt = ref(k)
	dt = 0.001 * xn(3,k)

	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine virsrc(xs,xr,xn,z0,xv,ier)
C  compute virtual source for a planar reflector, straight rays.
C  input parms:
C  xs(3) - shot location
C  xr(3) - rcvr location
C  xn(3) - unit normal vector for reflecting plane
C  z0 - z axis intercept
C  output parms:
C  xv(3) - virtual source
C  ier   - error parameter, normally 0. set to 1 if shot and rcvr
C          are on opposite sides of reflector.
C  the reflection of the shot location is determined and used as
C  a virtual source. the reflection point is the intersection of
C  the line from virtual source to receiver and the reflecting plane.
C
	dimension xn(3),xs(3),xr(3),xv(3)
C  distance from shot to plane (a), rcvr to plane (b)
	a=plndst(xs,z0,xn)
	b=plndst(xr,z0,xn)
C  check for shot and rcvr on opposite sides of plane
	if (a*b .gt. 0.) then
C  set virtual source
		d=2.*a
		xv(1)=xs(1)-d*xn(1)
		xv(2)=xs(2)-d*xn(2)
		xv(3)=xs(3)-d*xn(3)
		ier=0
	else
C  shot and rcvr on opposite sides of plane
		xv(1)=xr(1)
		xv(2)=xr(2)
		xv(3)=xr(3)
		ier = 1
	endif
	return
	end
C
C*----------------------------------------------------------------------
C
	function plndst(xs,z0,xn)
C  compute distance from point xs to plane with normal xn. z0 is the
C  z axis intercept of the plane.
	dimension xs(3),xn(3)
	plndst=xn(1)*xs(1) + xn(2)*xs(2) + xn(3)*(z0-xs(3))
	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine syntrc(n2,mag,nf,sf,amp,nref,dt,wt,z,wk)
C  synthetic trace generation
C  n2 - Fourier transform length
C  mag - transform magnitude (n2 = 2**mag)
C  sf - sample frequency in hz
C  amp - complex wavelet amplitude spectrum out to nyquist (n2/2 + 1)
C  nref - number of events
C  dt - event arrival times in seconds
C  wt - event relative amplitudes
C  z - output trace, length n2
C  wk - work array, complex length n2
	dimension dt(nref),wt(nref)
	complex amp(*),z(*),wk(*)
C  sample frequency in radians
	sfr=atan(1.)*8.*sf
C  Initialize
	call vfill( 0.0, z, 1, n2 + 2 )
C  Loop over events
	do j=1,nref
C  Add event if needed
		if (dt(j).gt.0.) 
     +    call dgen_accum(nf, sfr, dt(j), wt(j), amp, z, wk, wk(nf+1))
	enddo
C  Inverse FFT
	call seicrft(n2,z)
	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine dgen_accum( nf, sfr, dt, fact, amp, z, w1, w2 )
C
C  Accumulate phase shifted events
C
C  nf - number of frequencies
C  sfr - sample frequency in radians/sec
C  dt - travel time
C  fact - weighting factor
C  amp(nf) - amplitude array
C  z(nf) - input/output complex transform array
C  w1 - complex work array, length nf
C  w2 - real work array, length nf
C
	complex amp(nf), z(nf), w1(nf)
	real w2(nf)

C  Set phase values
	do i=1,nf
		w2(i) = dt*sfr*(1-i)
	enddo

C  Compute phase values
	call vsincos( w2, w1, nf )

C  Accumulate weighted values
	do i=1,nf
		z(i) = z(i) + fact * amp(i) * w1(i)
	enddo

	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine setnrm(xn,z0)
C  compute normal to reflector
C  xn(3) - input reflector dip, dip azimuth, and depth.
C          on output contains unit normal.
C  z0 - on output contains z axis intercept
	dimension xn(3)
C  convert angles to radians
	th=3.141592654*xn(1)/180.
	phi=3.141592654*xn(2)/180.
	sth=-sin(th)
C  Set z axis intercept
	z0 = xn(3)
C  set normal
	xn(1)=sth*cos(phi)
	xn(2)=sth*sin(phi)
	xn(3)=cos(th)
	return
	end
C
C*----------------------------------------------------------------------
C
	subroutine dgenamp(nyq,df,flc,flp,fhp,fhc,a)
C  compute gaussian bell amplitude spectrum
C  nyq - nyquist sample
C  df - sample frequency in hz
C  flc = Low cut frequency (-30 db point)
C  flp = low pass frequency
C  fhp = high pass frequency
C  fhc = high cut frequency (-30 db point)
C  a - output array, length n/2 + 1 containing amplitude spectrum
C
	dimension a(nyq)

C  Set gaussian parameters
	db = 30.
	dbmax = -10.*alog(10.)
	band = flp - flc
	alpha = -db*alog(10.)/20./(band/flp)**2

C  Set low side
	iflp = 1 + nint(flp/df)
	a(1) = 0.0
	do i=2,iflp-1
		f = df*(i-1)
		arg = alpha*((flp-f)/flp)**2
		if (arg.lt.dbmax) arg = dbmax
		a(i) = exp(arg)
	enddo

C  Center
	ifhp = 1 + nint(fhp/df)
	do i=iflp,ifhp
		a(i) = 1.0
	enddo

C  Now high side
	band = fhc - fhp
	alpha = -db*alog(10.)/20./(band/fhp)**2
	do i=ifhp+1,nyq
		f = df*(i-1)
		arg = alpha*((f-fhp)/fhp)**2
		if (arg.lt.dbmax) arg = dbmax
		a(i) = exp(arg)
	enddo

	return
	end

C*----------------------------------------------------------------------

	subroutine dgengho(nyq,sf,gap,amp,a,p)
C
C  generate zero phase ghost wavelet
C  nyq - nyquist sample
C  sf - sample frequency in hz
C  gap - gap between primary and ghost pulse, in seconds
C  amp - ghost amplitude (fraction of primary)
C  a - input real amplitude spectrum, length nyq
C  p - output complex wavelet length nyq
C
	dimension a(nyq)
	complex p(nyq)

C  Sample frequency in radians
	sw = 8.*atan(1.)*sf
C  Loop and sum primary and secondary pulses
	do i=1,nyq
C  Complex argument
		wt = sw*(1-i)*gap
		p(i) = a(i)*cmplx( 1.0 + amp*cos(wt), amp*sin(wt) )
	enddo

	return
	end

C*----------------------------------------------------------------------
