C*----------------------------------------------------------------------
C*  NAME:
C*    system.f - System support routines for seismic processing
C*               benchmark
C*     
C*----------------------------------------------------------------------
C*  DEPENDENCIES:
C*     sys.inc - seismic benchmark system common
C*
C*----------------------------------------------------------------------
C*  HISTORY:
C*
C*     Written:
C*       12/31/91 - C. Mosher, ARCO E&PT 
C*
C*     References:
C*       Seismic Benchmark Users Guide
C*
C*     Modified:
C*
C*----------------------------------------------------------------------

C*----------------------------------------------------------------------
C*  NAME:
C*    sysohdr - print header separating process output sections
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    character*4 name
C*    call sysohdr( name )
C*
C*----------------------------------------------------------------------

	subroutine sysohdr( nam )
	character*4 nam
	include 'sys.inc'
	character*80 buf
	
	call sysmsg(' ')
	write (buf,9010) nam
 9010 format (A4,' -------------------------------------------',
     +             '------------------------------')
	call sysmsg(buf)
	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    sysbcst - broadcast common block from master node
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    integer start, end
C*    call sysbcst( start, end )
C*
C*----------------------------------------------------------------------

	subroutine sysbcst( start, end )
	include 'para.inc'
	
C  Compute length 
	len = jloc( end ) - jloc( start ) + jwsize()

C  Broadcast from master node
	call jbrdcst( 1, start, len, master )
	
	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    syspflo - print processing flow
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    call syspflo( )
C*
C*----------------------------------------------------------------------

	subroutine syspflo( )
	include 'sys.inc'
	
	write (ipr,9010) (name(i),i=1,nproc)
 9010 format (/,' ARCO/Perfect Seismic Processing Benchmark',//,
     +    ' Processing Flow:',//,
     +    (2x,a4))

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    sysmsg - print informational message, master node only
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    character*(*) buf
C*    call sysmsg( buf )
C*
C*----------------------------------------------------------------------

	subroutine sysmsg( buf )
	character*(*) buf
	include 'sys.inc'
	include 'para.inc'
	
	if (node.ne.master) return
	write (ipr,*) buf

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    systrc - print trace message with process name and node number
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    character*(*) buf
C*    call systrc( buf )
C*
C*----------------------------------------------------------------------

	subroutine systrc( buf )
	character*(*) buf
	include 'sys.inc'
	include 'para.inc'
	
	write (ipr,'('' Node '',i3,'' Process '',a4)') node,name(jproc)
	write (ipr,*) buf

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    sysexit - Abnormal exit routine
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    character*(*) buf
C*    call sysexit( buf )
C*
C*----------------------------------------------------------------------

	subroutine sysexit( buf )
	character*(*) buf
	include 'sys.inc'
	include 'para.inc'
	
	write (ipr,9010) node
 9010 format (/,' *** SEIS ERROR: sysexit called by node ',i5)
	write (ipr,*) buf
	call jkill()
	call jpterm()
	call exit(-1)

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    sysabrt - Process ABORT routine
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    call sysabrt( )
C*
C*----------------------------------------------------------------------

	subroutine sysabrt( )
	include 'sys.inc'
	character*72 buf
	
	write (buf,9010) name(jproc)
 9010 format ('*** SEIS ERROR: Abort received from process ',a4)
	call sysexit( buf )

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    sysupd - Update system common block 
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    integer ldm, mxtrc
C*    call sysupd( ldm, mxtrc )
C*
C*----------------------------------------------------------------------

	subroutine sysupd( ldm, mxtrc )
	integer ldm, mxtrc
	include 'sys.inc'
	
C  Update leading dimension, max traces, reserved storage length,
C  and working storage length in system common

C  Intial values
	if (jproc.eq.1) then
		ldim = 1
		maxtrc = 1
		nra = 1
		nsa = 1
	endif

	ldim = max( ldm, ldim )
	maxtrc = max( mxtrc, maxtrc )

C  Set reserved storage pointer for the current process
	kra(jproc) = nra
	lenra(jproc) = lra
C  Increment reserved storage length
	nra = nra + lra
C  Make sure it's on a quad word boundary
	irem = mod(nra-1,4)
	if (irem.ne.0) nra = nra - irem + 4

C  Update scratch storage length
	nsa = max( nsa, lsa )

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    syschk - Print out memory requirements and return totals
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    call syschk( )
C*
C*----------------------------------------------------------------------

	subroutine syschk( )
	include 'sys.inc'
	include 'para.inc'
	
	call sysohdr('SEIS')

	write (ipr,9010) nodes,ldim,maxtrc,nra,nsa
 9010 format (/,' Seismic Processing Benchmark Resource Usage',//,
     +          ' No. Processors  : ',i8,/,
     +          ' Trace array size: ',2i8,/,
     +          ' Reserved storage: ',i8,/,
     +          ' Working storage : ',i8,/)

C  Return totals
	lotr = ldim*maxtrc
	lra = nra
	lsa = nsa

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    sysdone - Print execution time summary
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    call sysdone( )
C*
C*----------------------------------------------------------------------

	subroutine sysdone( )
	include 'sys.inc'
	include 'para.inc'
	character*12 hms

	call jsync()
	if (node.ne.master) then
		call jpterm()
		return
	endif

C*----------------------------------------------------------------------

	write (ipr,9010)
 9010 format (/,' ARCO/Perfect Seismic Benchmark Completed',//,
     +          ' Process    CPU Time (sec)')
	subtot = 0.0
	do i=1,nproc
		call hmstime( ctime(i), hms )
		write (ipr,9020) name(i),hms
 9020 	format(2x,a4,4x,a12)
		subtot = subtot + ctime(i)
	enddo

	total = timer(0)
	systim = total - subtot
	et0 = etimer(0) - et0

	write (ipr,*) ' '
	call hmstime(systim,hms)
	write (ipr,9030) 'System ',hms
	call hmstime(total,hms)
	write (ipr,9030) 'Total  ',hms
	write (ipr,*) ' '
	call hmstime(et0,hms)
	write (ipr,9030) 'Elapsed',hms
 9030 format (1x,a7,2x,a12)

	write (ipr,9040)
 9040 format(/,' --------------------------------------------------',
     + '--------------------')

	call jpterm()

	return
	end

C*----------------------------------------------------------------------

	subroutine hmstime( time, hms )
C
C  Convert time in seconds to HH:MM:SS.S string
C
	real time,s
	integer ih,im,is
	character*12 hms
	is = int(time)
	ih = is/3600
	is = is - 3600*ih
	im = is/60
	s = time - float(3600*ih) - float(60*im)
	write (hms,9010) ih,im,s
 9010 format (i4,':',i2,':',f4.1)
	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    syspars - Parse 'PROC' parameter record for process names
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    call syspars( )
C*
C*----------------------------------------------------------------------

	subroutine syspars( )
	include 'sys.inc'

	character*80 data
	character*4 pnam
	character*1 c,blank,tab,newlin,null,toupper

	blank = ' '
	tab = char(9)
	newlin = char(13)
	null = char(0)	

c  set number of processes to zero
	nproc = 0
	index = 0

   5  continue
	index = index + 1
	call getparm('PROC',index,data,ier)
	if (ier.ne.0) return
c  start at character 5 (skip over PROC) and look for process names
	i = 4
  10  continue
		i = i+1
		if (i.gt.80) goto 5
		c = data(i:i)
c  quit if null or newlin
		if (c.eq.null .or. c.eq.newlin) goto 5
c  skip over white space
		if (c.eq.blank .or. c.eq.tab) goto 10
c  break out process name
		if (i.gt.76) goto 5
		nproc = nproc+1
		do 20 j=1,4
			c = toupper(data(i:i))
			i = i+1
			if ( (c.ge.'A' .and. c.le.'Z') .or. 
     +			 (c.ge.'0' .and. c.le.'9') ) then
				pnam(j:j) = c
			else
				write (*,
     +    '('' ***SEIS ERROR: Invalid process name'')')
				call sysexit('*** SEIS Terminating ...')
			endif
  20  	continue
		name(nproc) = pnam
		goto 10
c
	end

C*----------------------------------------------------------------------
C
C  character*1 toupper(c) - convert char*1 argument to upper case
C		
	character*1 function toupper(c)
	character*1 c
	if (c.ge.'a' .and. c.le.'z') then
		toupper = char(ichar(c) - 32)
	else
		toupper = c
	endif
	return
	end

C*----------------------------------------------------------------------
C*
C*  NAME:
C*    sysparm - get parameter=keyword value from seismic parameter file
C*     
C*  SYNOPSIS:
C*    character*(*) keyword
C*    character*1 type
C*    (character | integer | real) value
C*    integer ier
C*    call sysparm( keyword, type, value, ier )
C*
C*  DESCRIPTION: 
C*    Searches for seismic parameter file records with first four
C*    characters that match process name, then attempts to locate 
C*    'keyword=value' string.  Value is decoded according to type,
C*    'I'=integer, 'F'=float or real, 'C'=character string.
C*    Returns ier=0 on success, non-zero for no match.
C*
C*----------------------------------------------------------------------
C*   DEPENDENCIES:
C*
C*     Include files:
C*       ./sys.inc
C*
C*     Libraries:
C*       FORTRAN I/O support
C*
C*----------------------------------------------------------------------
C*   HISTORY:
C*
C*      Written:
C*        12/31/91 - C. Mosher, ARCO E&PT 
C*
C*      References:
C*
C*      Modified:
C*
C*----------------------------------------------------------------------
C
	subroutine sysparm( keyword, type, value, ier )
C  Arguments
	character*(*) keyword
	character*1 type
	integer value
C  Local variables
	character*80 buf
	integer ier, sfetch, isfetch, rsfetch
	include 'sys.inc'

C Rewind paramter file
	rewind( iuparm )

C  Loop and read paramter file records for the current process
  10  continue
	read ( iuparm, '(a80)', iostat=ier ) buf
	if (ier.ne.0) return
	if (buf(1:4).ne.name(jproc)) goto 10

C  Attempt to parse 'keyword=value' from current record
	if (type.eq.'C') then
		ier = sfetch( buf(5:80), keyword, value )
		if (ier.gt.0) then
			ier = 0
		else
			ier = -1
		endif
	else if (type.eq.'I') then
		ier = isfetch( buf(5:80), keyword, value )
	else if (type.eq.'F') then
		ier = rsfetch( buf(5:80), keyword, value )
	else
		ier = -1
	endif

C  Loop if no match found
	if (ier.ne.0) goto 10

	return
	end

C*----------------------------------------------------------------------
C*  NAME:
C*    syspocr - Return current, first, and last occurence 
C*              of process 'NAME'
C*     
C*----------------------------------------------------------------------
C*  SYNOPSIS:
C*
C*    character*4 name
C*    integer iproc, iproc1, iprocn
C*    call syspocr( name, iproc, iproc1, iprocn )
C*
C*----------------------------------------------------------------------

	subroutine syspocr( pname, iproc, iproc1, iprocn )
C
	character*4 pname
	integer iproc, iproc1, iprocn
	include 'sys.inc'

	iproc = jproc
	iproc1 = jproc
	iprocn = jproc
	do i=jproc-1,1,-1
		if (name(i).eq.pname) iproc1 = i
	enddo
	do i=jproc+1,nproc
		if (name(i).eq.pname) iprocn = i
	enddo
		
	return
	end
