/*--------------------------------------------------------------------*/
/*  NAME:
/*    fetch_f.c - Fortran interfaces to parameter keyword file I/O
/*
/*  VERSION:
/*    Initial version with Cray 'fcd' macros
/*
/*  Routines defined in this module:
/*    opnprm - open paramter file
/*    clsprm - close paramter file
/*    ifetch - return integer from matching keyword
/*    rfetch - return float (real) from matching keyword
/*    fetch - return character string from matching keyword
/*     
/*--------------------------------------------------------------------*/
/*
/*  HISTORY:
/*
/*     Written:
/*       10/08/93 - C. Mosher, ARCO E&PT 
/*
/*     References:
/*
/*     Modified:
/*
/*--------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>

/* Detect both default Cray and PVM Cray ARCH strings */
#ifdef cray
#define ISCRAY
#endif
#ifdef CRAY
#define ISCRAY
#endif

/* Define Cray versions of Fortran external names */
#ifdef ISCRAY
#define opnprm_ OPNPRM
#define clsprm_ CLSPRM
#define fetch_ FETCH
#define sfetch_ SFETCH
#define ifetch_ IFETCH
#define isfetch_ ISFETCH
#define rfetch_ RFETCH
#define rsfetch_ RSFETCH
#define lenstrw_ LENSTRW
#include "fortran.h"
#endif


/*--------------------------------------------------------------------*/
/* Fortran interfaces
/*--------------------------------------------------------------------*/

int opnprm_(dsn,len_dsn)
/*
	Open parm file - FORTRAN interface
*/
char *dsn;
int len_dsn;
{
	FILE *fp,*fopen();
	char *p, *fstrnwcp();

	p = fstrnwcp( dsn, len_dsn );
	fp = fopen( p, "r");

	free(p);
	return((int)fp);
}
 

/*--------------------------------------------------------------------*/

int clsprm_(ifp)
/*
	Close parm file - FORTRAN interface
*/
int *ifp;
{
	FILE *fp;

	fp = (FILE *)(*ifp);
	return( close(fp) );
}
 
/*--------------------------------------------------------------------*/

int fetch_(ifp,keyword,string,len_k,len_s)
/*
	Fetch keyword string from parm file
*/
int *ifp;
char *keyword;
char *string;
int len_k, len_s;
{
	FILE *fp;
	char *p, *fstrnwcp();
	int iret;
	
	fp = (FILE *)(*ifp);
	p = fstrnwcp( keyword, len_k );
	iret = fetch( fp, p, string );
	free(p);
	return(iret);
}
 
/*--------------------------------------------------------------------*/

int sfetch_(buf,key,string,len_b,len_k,len_s)
/*
	Fetch keyword string from character buffer
*/
char *buf;
char *key;
char *string;
int len_b, len_k, len_s;
{
	FILE *fp;
	char *buf_c, *key_c, *fstrnwcp(), *fstrncpy();
	int iret;
	
	buf_c = fstrncpy( buf, len_b );
	key_c = fstrnwcp( key, len_k );
	iret = sfetch( buf_c, key_c, string );
	free(key_c);
	free(buf_c);
	return(iret);
}
 
/*--------------------------------------------------------------------*/

int ifetch_(ifp,keyword,value,len_k)
/*
	Fetch keyword integer from parm file
*/
int *ifp;
char *keyword;
int *value;
int len_k;
{
	FILE *fp;
	char *p, *fstrnwcp();
	int iret;
	
	fp = (FILE *)(*ifp);
	p = fstrnwcp( keyword, len_k );
	iret = ifetch( fp, p, value );
	free(p);
	return(iret);
}
 
/*--------------------------------------------------------------------*/

int isfetch_(buf,key,ival,len_b,len_k)
/*
	Fetch keyword integer from character buffer
*/
char *buf;
char *key;
int *ival;
int len_b, len_k;
{
	FILE *fp;
	char *buf_c, *key_c, *fstrnwcp(), *fstrncpy();
	int iret;
	
	buf_c = fstrncpy( buf, len_b );
	key_c = fstrnwcp( key, len_k );
	iret = isfetch( buf_c, key_c, ival );
	free(key_c);
	free(buf_c);
	return(iret);
}

/*--------------------------------------------------------------------*/

int rfetch_(ifp,keyword,value,len_k)
/*
	Fetch keyword float from parm file
*/
int *ifp;
char *keyword;
float *value;
int len_k;
{
	FILE *fp;
	char *p, *fstrnwcp();
	int iret;
	
	fp = (FILE *)(*ifp);
	p = fstrnwcp( keyword, len_k );
	iret = rfetch( fp, p, value );
	free(p);
	return(iret);
}
 
/*--------------------------------------------------------------------*/

int rsfetch_(buf,key,val,len_b,len_k)
/*
	Fetch keyword float from character buffer
*/
char *buf;
char *key;
float *val;
int len_b, len_k;
{
	FILE *fp;
	char *buf_c, *key_c, *fstrnwcp(), *fstrncpy();
	int iret;
	
	buf_c = fstrncpy( buf, len_b );
	key_c = fstrnwcp( key, len_k );
	iret = rsfetch( buf_c, key_c, val );
	free(key_c);
	free(buf_c);
	return(iret);
}

/*--------------------------------------------------------------------*/

char *fstrncpy( string, len )

/*--------------------------------------------------------------------*/
/*   Return pointer to a null terminated 'C' string copied from
/*   a fortran string
/*--------------------------------------------------------------------*/

char *string;		/* input string */
int len;			/* Max string length */
{
	char *p,*malloc();
	
#ifdef ISCRAY
	len = _fcdlen((_fcd)string);
	p = malloc(len+1);
	strncpy( p, _fcdtocp((_fcd)string), len );
#else
	p = malloc(len+1);
	strncpy(p,string,len);
#endif
	p[len] = '\0';

	return( p );
}
 
/*--------------------------------------------------------------------*/

int lenstrw_( s, n, len_s )
/*
	Return length of string up to first white space
*/
char *s;
int *n;
int len_s;
{
	char *s_c, *fstrnwcp();
	int iret;

	s_c = fstrnwcp( s, len_s );
	iret = lenstrw( s_c, *n );
	free( s_c );
	return( iret );
}

/*--------------------------------------------------------------------*/

char *fstrnwcp( string, len )

/*--------------------------------------------------------------------*/
/*   Return pointer to a null terminated 'C' string copied from
/*   a fortran string, null terminated at first white space character,
/*   or at the end of the string.
/*--------------------------------------------------------------------*/

char *string;		/* input string */
int len;			/* Max string length */
{
	char *p,*malloc();
	
#ifdef ISCRAY
	len = _fcdlen((_fcd)string);
	p = malloc(len+1);
	strnwcp( p, _fcdtocp((_fcd)string), len );
#else
	p = malloc(len+1);
	strnwcp(p,string,len);
#endif

	return( p );
}
