/*-------------------------------------------------------------------*/
/*
	Machine dependent system routines
	Default version, with support for BSD, SYSV, and SOL2 systems
	Define '-DBSD', -DSYSV', or '-DSOL2' when compiling.
*/
/*-------------------------------------------------------------------*/

#ifdef SYSV

#include <sys/types.h>
#include <sys/times.h>
#include <sys/param.h>
#include <limits.h>

int cputime_()
/*
	Return current total CPU time used by process in milliseconds.
*/
{
	int it;
	struct tms buf;

	times( &buf );
/* Get time in clock ticks */
	it = ( buf.tms_utime + buf.tms_stime + buf.tms_cutime + 
		buf.tms_cstime );
/* Convert to milliseconds */
	return( (int)( 1000.*(float)(it)/(float)(CLK_TCK) ) );
}

int sysclock_()
/*
	Return current system clock time in milliseconds
*/
{
	struct tms tb;
	clock_t it, times();

/* Get time in clock ticks */
	it = times( &tb );
/* Convert to milliseconds */
	return( (int)( 1000.*(float)(it)/(float)(CLK_TCK) ) );
}

int jwsize_()
{
	return(sizeof(int));
}

int jloc_( loc )
int *loc;
{
	return( (int)(loc) );
}

#endif

/*-------------------------------------------------------------------*/

#ifdef BSD

#include <sys/types.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <sys/resource.h>

int cputime_()
/*
	Return current total CPU time used by process in milliseconds
*/
{
	struct rusage ru;

	getrusage( RUSAGE_SELF, &ru );
	
	return( (ru.ru_utime.tv_sec  + ru.ru_stime.tv_sec ) * 1000 +
			(ru.ru_utime.tv_usec + ru.ru_stime.tv_usec) / 1000 );
}

int sysclock_()
/*
	Return current system clock time in milliseconds
*/
{
	struct timeb tb;

	ftime( &tb );
	return ( (1000 * tb.time) + tb.millitm );
}

int jwsize_()
{
	return(sizeof(int));
}

int jloc_( loc )
int *loc;
{
	return( (int)(loc) );
}

#endif

/*-------------------------------------------------------------------*/
