/*--------------------------------------------------------------------*/
/*  FILE:
/*    memflush - set memory to constant value
/*  
/*  USAGE:
/*    memflush nwords [value]
/*      where nwords is the number of words of memory to set
/*      and value is the optional value to set, default 0xdeadbeaf
/*
/*  DESCRIPTION:
/*    Used to clean a large block of memory so that subsequent I/O
/*    requests are served from disk rather than cached blocks in memory.
/*
/*--------------------------------------------------------------------*/

main(argc,argv)
int argc;
char *argv[];
{
	int i, nwords, nvals;
	unsigned value;
	int *p1, *p2, *p3, *p4;
	if (argc < 2) {
		printf("Usage: memflush nwords [value]\n");
		exit(1);
	}

	sscanf(argv[1],"%d",&nwords);
	nvals = nwords/4;
	nwords = 4*nvals;
	printf("Flush %d words in memory\n",nwords);
	if (nvals==0) exit(0);
	if (argc > 2 ) 
		sscanf(argv[2],"%u",&value);
	else
		value = 0xdeadbeaf;

	p1 = (int *)malloc( nwords );
	p2 = (int *)malloc( nwords );
	p3 = (int *)malloc( nwords );
	p4 = (int *)malloc( nwords );

	if (p1 == (int *)0 ||
		p2 == (int *)0 ||
		p3 == (int *)0 ||
		p4 == (int *)0 ) {
			printf("Memory allocation failed\n");
			perror("malloc");
			exit(1);
	}

	for (i=0; i<nvals; i++)
		p1[i] = p2[i] = p3[i] = p4[i] = value;

	printf("memflush complete\n");
	exit(0);
}
