C*--------------------------------------------------------------------
C*  File:
C*    sbfsubs.f - Fortran support for Seismic Benchmark File I/O
C*
C*  Routines defined in this module:
C*    sbfcopy - memory to memory frame copy
C*
C*--------------------------------------------------------------------

C*--------------------------------------------------------------------
C*  NAME:
C*    sbfcopy - Memory to memory frame copy
C*     
C*  SYNOPSIS:
C*    integer n1a, n2a, n1b, n2b;
C*    real a(n1a,n2a), b(n1b,n2b)
C*    call sbfcopy( n1a, n2a, a, n1b, n2b, b );
C*
C*  DESCRIPTION:
C*    Copies a frame from one 2D array to another, possibly in-place.
C*    Sets a(i,j) = b(i,j), truncating 'b' if destination 'a' is 
C*    smaller, zero-fills if destination is larger.
C*
C*--------------------------------------------------------------------

	subroutine sbfcopy( n1a, n2a, a, n1b, n2b, b )
	implicit none
C  Arguments
	integer n1a,n2a,n1b,n2b
	real a(n1a,n2a), b(n1b,n2b)
C  Local declarations
	integer n1, n2, i, j
	integer jloc, ja, jb

C  Set minimum copy range
	n1 = min(n1a,n1b)
	n2 = min(n2a,n2b)

C  Get start address of arrays
	ja = jloc(a)
	jb = jloc(b)

C  Back to front copy if input/output are the same and the destination
C    has a larger leading dimension
	if (ja.eq.jb .and. n1a.gt.n1b) then
		do j=n2,1,-1
			do i=n1,1,-1
				a(i,j) = b(i,j)
			enddo
		enddo
C  Otherwise, front to back copy
	else
		do j=1,n2
			do i=1,n1
				a(i,j) = b(i,j)
			enddo
		enddo
	endif
	
C  Zero pad end of rows if destination is larger
	if (n1a.gt.n1b) then
		do j=1,n2
			do i=n1+1,n1a
				a(i,j) = 0.0
			enddo
		enddo
	endif


C  Zero pad columns if destination is larger
	if (n2a .gt. n2b) then
		do j=n2b+1,n2a
			do i=1,n1a
				a(i,j) = 0.0
			enddo
		enddo
	endif

	return
	end
